@@ -1,4 +1,4 @@
-# $Id: IRC-Qnet-State.pm,v 3.7 2005/03/01 17:43:16 chris Exp $
+# $Id: IRC-Qnet-State.pm,v 3.8 2005/03/04 17:37:21 chris Exp $
 #
 # POE::Component::IRC::Qnet::State, by Chris Williams
 #
@@ -211,6 +211,37 @@ sub _create {
   return $self;
 }
 
+sub _parseline {
+  my ($session, $self, $line) = @_[SESSION, OBJECT, ARG0];
+  my (@events, @cooked);
+
+  # Feed the proper Filter object the raw IRC text and get the
+  # "cooked" events back for sending, then deliver each event. We
+  # handle CTCPs separately from normal IRC messages here, to avoid
+  # silly module dependencies later.
+
+  @cooked = ($line =~ tr/\001// ? @{$self->{ctcp_filter}->get( [$line] )}
+             : @{$self->{irc_filter}->get( [$line] )} );
+
+  foreach my $ev (@cooked) {
+    if ( $ev->{name} eq 'part' and not $self->{'dont_partfix'} ) {
+        (@{$ev->{args}}[1..2]) = split(/ /,$ev->{args}->[1],2);
+        $ev->{args}->[2] =~ s/^:// if ( defined ( $ev->{args}->[2] ) );
+    }
+    # If its 001 event grab the server name and stuff it into {INFO}
+    if ( $ev->{name} eq '001' ) {
+        $self->{INFO}->{ServerName} = $ev->{args}->[0];
+        # Kind of assuming that $line is a single line of IRC protocol.
+        $self->{RealNick} = ( split / /, $line )[2];
+    }
+    if ( $ev->{name} eq 'nick' or $ev->{name} eq 'quit' ) {
+        push ( @{$ev->{args}}, [ $self->nick_channels( ( split /!/, $ev->{args}->[0] )[0] ) ] );
+    }
+    $ev->{name} = 'irc_' . $ev->{name};
+    $self->_send_event( $ev->{name}, @{$ev->{args}} );
+  }
+}
+
 # Qnet extension to RPL_WHOIS
 sub irc_330 {
   my ($kernel,$self) = @_[KERNEL,OBJECT];
@@ -258,14 +289,14 @@ sub irc_315 {
     $self->_channel_sync_who($channel);
     if ( $self->_channel_sync($channel) ) {
         delete ( $self->{CHANNEL_SYNCH}->{ u_irc ( $channel ) } );
-        $self->_send_event( $kernel, 'irc_chan_sync', $channel );
+        $self->_send_event( 'irc_chan_sync', $channel );
     }
   # Otherwise we assume its a nickname
   } else {
 	if ( defined ( $self->{USER_AUTHED}->{ u_irc ( $channel ) } ) ) {
-	   $self->_send_event( $kernel, 'irc_nick_authed', $channel, delete ( $self->{USER_AUTHED}->{ u_irc ( $channel ) } ) );
+	   $self->_send_event( 'irc_nick_authed', $channel, delete ( $self->{USER_AUTHED}->{ u_irc ( $channel ) } ) );
 	} else {
-           $self->_send_event( $kernel, 'irc_nick_sync', $channel );
+           $self->_send_event( 'irc_nick_sync', $channel );
 	}
   }
 }
